# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['shellcode']

package_data = \
{'': ['*']}

install_requires = \
['capstone>=4.0.2,<5.0.0', 'typer>=0.3.2,<0.4.0']

setup_kwargs = {
    'name': 'shellcode',
    'version': '0.1.3',
    'description': 'CLI to turn shellcode back to asm.',
    'long_description': '# shellcode\n![shellcode logo](https://raw.githubusercontent.com/4thel00z/shellcode/master/logo.png)\n\n## Motivation\n\nIn the course of pawning n00bs often the need arises to introspect some (generated) shellcode or check\nit for sanity.\n\nThis tools does exactly that.\n\n## Usage\n```\necho "<some shellcode>" | python3 -m shellcode \n```\n\n## License\n\nThis project is licensed under the GPL-3 license.\n',
    'author': '4thel00z',
    'author_email': '4thel00z@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/4thel00z/shellcode',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
